@@ -15,6 +15,11 @@ from scipy.sparse.linalg import lobpcg as sp_lobpcg
 from scipy.sparse import csr_matrix
 
 from pylops.utils.backend import get_array_module, get_module, get_sparse_eye
+from pylops.utils.estimators import (
+    trace_hutchinson,
+    trace_hutchpp,
+    trace_nahutchpp,
+)
 from pylops.optimization.solver import cgls
 
 logging.basicConfig(format='%(levelname)s: %(message)s', level=logging.WARNING)
@@ -44,12 +49,15 @@ class LinearOperator(spLinearOperator):
         (``True``) or not (``False``)
 
     """
-    def __init__(self, Op=None, explicit=False):
+    def __init__(self, Op=None, explicit=False, clinear=None):
         self.explicit = explicit
         if Op is not None:
             self.Op = Op
             self.shape = self.Op.shape
             self.dtype = self.Op.dtype
+            self.clinear = getattr(self.Op, 'clinear', True)
+        if clinear is not None:
+            self.clinear = clinear
 
     def _matvec(self, x):
         if callable(self.Op._matvec):
@@ -71,8 +79,6 @@ class LinearOperator(spLinearOperator):
             y = np.vstack([self.matvec(col.reshape(-1)) for col in X.T]).T
         return y
 
-        return np.vstack([self.matvec(col.reshape(-1)) for col in X.T]).T
-
     def __mul__(self, x):
         y = super().__mul__(x)
         if isinstance(y, spLinearOperator):
@@ -240,11 +246,15 @@ class LinearOperator(spLinearOperator):
 
         """
         if isinstance(x, LinearOperator):
-            return _ProductLinearOperator(self, x)
+            Op = _ProductLinearOperator(self, x)
+            # Output is C-Linear only if both operators are
+            Op.clinear = (getattr(self, 'clinear', True)
+                          and getattr(x, 'clinear', True))
+            return Op
         elif np.isscalar(x):
             return _ScaledLinearOperator(self, x)
         else:
-            if x.ndim == 1 or x.ndim == 2 and x.shape[1] == 1:
+            if x.ndim == 1:# or x.ndim == 2 and x.shape[1] == 1:
                 return self.matvec(x)
             elif x.ndim == 2:
                 return self.matmat(x)
@@ -482,7 +492,7 @@ class LinearOperator(spLinearOperator):
                                          'for non-symmetric square matrices of '
                                          'complex type...')
                     if symmetric and uselobpcg:
-                        X = np.random.rand(self.shape[0], neigs)
+                        X = np.random.rand(self.shape[0], neigs).astype(self.dtype)
                         eigenvalues = \
                             sp_lobpcg(self.A, X=X, maxiter=niter,
                                       **kwargs_eig)[0]
@@ -499,7 +509,7 @@ class LinearOperator(spLinearOperator):
                                                          self.A)))
                 else:
                     if uselobpcg:
-                        X = np.random.rand(self.shape[1], neigs)
+                        X = np.random.rand(self.shape[1], neigs).astype(self.dtype)
                         eigenvalues = np.sqrt(sp_lobpcg(
                             np.dot(np.conj(self.A.T), self.A),
                             X=X, maxiter=niter, **kwargs_eig)[0])
@@ -516,7 +526,7 @@ class LinearOperator(spLinearOperator):
                                      'non symmetric square matrices of '
                                      'complex type...')
                 if symmetric and uselobpcg:
-                    X = np.random.rand(self.shape[0], neigs)
+                    X = np.random.rand(self.shape[0], neigs).astype(self.dtype)
                     eigenvalues = \
                         sp_lobpcg(self, X=X, maxiter=niter, **kwargs_eig)[0]
                 elif symmetric:
@@ -527,7 +537,7 @@ class LinearOperator(spLinearOperator):
                                           **kwargs_eig)[0]
             else:
                 if uselobpcg:
-                    X = np.random.rand(self.shape[1], neigs)
+                    X = np.random.rand(self.shape[1], neigs).astype(self.dtype)
                     eigenvalues = np.sqrt(sp_lobpcg(self.H * self, X=X,
                                                     maxiter=niter,
                                                     **kwargs_eig)[0])
@@ -666,6 +676,82 @@ class LinearOperator(spLinearOperator):
         imagop = _RealImagLinearOperator(self, forw, adj, False)
         return imagop
 
+    def trace(
+        self,
+        neval=None,
+        method=None,
+        backend="numpy",
+        **kwargs_trace,
+    ):
+        r"""Trace of linear operator.
+
+        Returns the trace (or its estimate) of the linear operator.
+
+        Parameters
+        ----------
+        neval : :obj:`int`, optional
+            Maximum number of matrix-vector products compute. Default depends
+            ``method``.
+        method : :obj:`str`, optional
+            Should be one of the following:
+
+                - **explicit**: If the operator is not explicit, will convert to
+                  dense first.
+                - **hutchinson**: see :obj:`pylops.utils.trace_hutchinson`
+                - **hutch++**: see :obj:`pylops.utils.trace_hutchpp`
+                - **na-hutch++**: see :obj:`pylops.utils.trace_nahutchpp`
+
+            Defaults to 'explicit' for explicit operators, and 'Hutch++' for
+            the rest.
+        backend : :obj:`str`, optional
+            Backend used to densify matrix (``numpy`` or ``cupy``). Note that
+            this must be consistent with how the operator has been created.
+        **kwargs_trace
+            Arbitrary keyword arguments passed to
+            :obj:`pylops.utils.trace_hutchinson`,
+            :obj:`pylops.utils.trace_hutchpp`, or
+            :obj:`pylops.utils.trace_nahutchpp`
+
+        Returns
+        -------
+        trace : :obj:`self.dtype`
+            Operator trace.
+
+        Raises
+        -------
+        ValueError
+             If the operator has rectangular shape (``shape[0] != shape[1]``)
+
+        NotImplementedError
+            If the ``method`` is not one of the available methods.
+        """
+        if self.shape[0] != self.shape[1]:
+            raise ValueError("operator is not square.")
+
+        ncp = get_module(backend)
+
+        if method is None:
+            method = "explicit" if self.explicit else "hutch++"
+
+        method_l = method.lower()
+        if method_l == "explicit":
+            A = self.A if self.explicit else self.todense(backend=backend)
+            return ncp.trace(A)
+        elif method_l == "hutchinson":
+            return trace_hutchinson(
+                self, neval=neval, backend=backend, **kwargs_trace
+            )
+        elif method_l == "hutch++":
+            return trace_hutchpp(
+                self, neval=neval, backend=backend, **kwargs_trace
+            )
+        elif method_l == "na-hutch++":
+            return trace_nahutchpp(
+                self, neval=neval, backend=backend, **kwargs_trace
+            )
+        else:
+            raise NotImplementedError(f"method {method} not available.")
+
 
 def _get_dtype(operators, dtypes=None):
     if dtypes is None:
@@ -760,7 +846,7 @@ class _ColumnLinearOperator(LinearOperator):
 
     def _rmatvec(self, x):
         if self.explicit:
-            y = self.Opcol.T @ x
+            y = self.Opcol.T.conj() @ x
         else:
             y = self.Op._rmatvec(x)
             y = y[self.cols]
