@@ -134,7 +134,7 @@ def _hardthreshold_percentile(x, perc):
 
     """
     thresh = np.percentile(np.abs(x), perc)
-    return _halfthreshold(x, 0.5 * thresh ** 2)
+    return _hardthreshold(x, 0.5 * thresh ** 2)
 
 def _softthreshold_percentile(x, perc):
     r"""Percentile Soft thresholding.
@@ -256,7 +256,7 @@ def _IRLS_model(Op, data, nouter, threshR=False, epsR=1e-10,
         data = data - Op * x0
     if returnhistory:
         xinv_hist = ncp.zeros((nouter + 1, int(Op.shape[1])))
-        rw_hist = ncp.zeros((nouter + 1, int(Op.shape[0])))
+        rw_hist = ncp.zeros((nouter + 1, int(Op.shape[1])))
 
     Iop = Identity(data.size, dtype=data.dtype)
     # first iteration (unweighted least-squares)
@@ -409,7 +409,7 @@ def IRLS(Op, data, nouter, threshR=False, epsR=1e-10,
         \mathbf{d} = \mathbf{Op} \mathbf{x}
 
     by a set of outer iterations which require to repeatedly solve a
-    weighted least squares problem of the form:
+    weighted least squares problem of the form [2]_:
 
     .. math::
         \mathbf{x}^{(i+1)} = \operatorname*{arg\,min}_\mathbf{x}
@@ -417,8 +417,8 @@ def IRLS(Op, data, nouter, threshR=False, epsR=1e-10,
         \mathbf{d} = \mathbf{Op} \mathbf{x}
 
     where :math:`\mathbf{R}^{(i)}` is a diagonal weight matrix
-    whose diagonal elements at iteration :math:`i` are equal to the absolute
-    inverses of the model vector :math:`\mathbf{x}^{(i)}` at iteration
+    whose diagonal elements at iteration :math:`i` are equal to the absolutes
+    of the model vector :math:`\mathbf{x}^{(i)}` at iteration
     :math:`i`. More specifically the j-th element of the diagonal of
     :math:`\mathbf{R}^{(i)}` is
 
@@ -426,6 +426,8 @@ def IRLS(Op, data, nouter, threshR=False, epsR=1e-10,
         R^{(i)}_{j,j} = |x^{(i)}_j|
 
     .. [1] https://en.wikipedia.org/wiki/Iteratively_reweighted_least_squares
+    .. [2] Chartrand, R., and Yin, W. "Iteratively reweighted algorithms for
+       compressive sensing", IEEE. 2008.
 
     """
     if kind == 'data':
@@ -493,7 +495,7 @@ def OMP(Op, data, niter_outer=10, niter_inner=40, sigma=1e-4,
 
     .. math::
             ||\mathbf{x}||_0 \quad  subj. to \quad
-            ||\mathbf{Op}\mathbf{x}-\mathbf{b}||_2^2 <= \sigma,
+            ||\mathbf{Op}\mathbf{x}-\mathbf{b}||_2^2 <= \sigma^2,
 
     using Orthogonal Matching Pursuit (OMP). This is a very
     simple iterative algorithm which applies the following step:
@@ -501,7 +503,7 @@ def OMP(Op, data, niter_outer=10, niter_inner=40, sigma=1e-4,
     .. math::
         \Lambda_k = \Lambda_{k-1} \cup \{ arg max_j
         |\mathbf{Op}_j^H \mathbf{r}_k| \} \\
-        \mathbf{x}_k =  \{ arg min_{\mathbf{x}}
+        \mathbf{x}_k = arg min_{\mathbf{x}}
         ||\mathbf{Op}_{\Lambda_k} \mathbf{x} - \mathbf{b}||_2^2
 
     Note that by choosing ``niter_inner=0`` the basic Matching Pursuit (MP)
@@ -609,10 +611,10 @@ def OMP(Op, data, niter_outer=10, niter_inner=40, sigma=1e-4,
 
 def ISTA(Op, data, niter, eps=0.1, alpha=None, eigsiter=None, eigstol=0,
          tol=1e-10, monitorres=False, returninfo=False, show=False,
-         threshkind='soft', perc=None, callback=None):
+         threshkind='soft', perc=None, callback=None, decay=None, SOp=None, x0=None):
     r"""Iterative Shrinkage-Thresholding Algorithm (ISTA).
 
-    Solve an optimization problem with :math:`Lp, \quad p=0, 1/2, 1`
+    Solve an optimization problem with :math:`L_p, \; p=0, 1/2, 1`
     regularization, given the operator ``Op`` and data ``y``. The operator
     can be real or complex, and should ideally be either square :math:`N=M`
     or underdetermined :math:`N<M`.
@@ -654,7 +656,11 @@ def ISTA(Op, data, niter, eps=0.1, alpha=None, eigsiter=None, eigstol=0,
     callback : :obj:`callable`, optional
         Function with signature (``callback(x)``) to call after each iteration
         where ``x`` is the current model vector
-
+    decay : :obj:`numpy.ndarray`, optional
+        Decay factor to be applied to thresholding during iterations
+    SOp : :obj:`pylops.LinearOperator`, optional
+        Regularization operator (use when solving the analysis problem)
+    
     Returns
     -------
     xinv : :obj:`numpy.ndarray`
@@ -684,14 +690,23 @@ def ISTA(Op, data, niter, eps=0.1, alpha=None, eigsiter=None, eigstol=0,
 
     Notes
     -----
-    Solves the following optimization problem for the operator
+    Solves the following synthesis problem for the operator
     :math:`\mathbf{Op}` and the data :math:`\mathbf{d}`:
 
     .. math::
         J = ||\mathbf{d} - \mathbf{Op} \mathbf{x}||_2^2 +
             \epsilon ||\mathbf{x}||_p
 
-    using the Iterative Shrinkage-Thresholding Algorithms (ISTA) [1]_, where
+    or the analysis problem:
+
+    .. math::
+        J = ||\mathbf{d} - \mathbf{Op} \mathbf{x}||_2^2 +
+            \epsilon ||\mathbf{SOp}^H\mathbf{x}||_p
+
+    if ``SOp`` is provided. Note that in the first case, ``SOp`` should be
+    assimilated in the modelling operator (i.e., ``Op=GOp * SOp``).
+
+    The Iterative Shrinkage-Thresholding Algorithms (ISTA) [1]_ is used, where
     :math:`p=0, 1, 1/2`. This is a very simple iterative algorithm which
     applies the following step:
 
@@ -699,6 +714,13 @@ def ISTA(Op, data, niter, eps=0.1, alpha=None, eigsiter=None, eigstol=0,
         \mathbf{x}^{(i+1)} = T_{(\epsilon \alpha /2, p)} (\mathbf{x}^{(i)} +
         \alpha \mathbf{Op}^H (\mathbf{d} - \mathbf{Op} \mathbf{x}^{(i)}))
 
+    or
+
+    .. math::
+        \mathbf{x}^{(i+1)} = \mathbf{SOp}(T_{(\epsilon \alpha /2, p)}
+        (\mathbf{SOp}^H(\mathbf{x}^{(i)} + \alpha \mathbf{Op}^H (\mathbf{d} -
+        \mathbf{Op} \mathbf{x}^{(i)}))))
+
     where :math:`\epsilon \alpha /2` is the threshold and :math:`T_{(\tau, p)}`
     is the thresholding rule. The most common variant of ISTA uses the
     so-called soft-thresholding rule :math:`T(\tau, p=1)`. Alternatively an
@@ -741,6 +763,10 @@ def ISTA(Op, data, niter, eps=0.1, alpha=None, eigsiter=None, eigstol=0,
     # identify backend to use
     ncp = get_array_module(data)
 
+    # prepare decay (if not passed)
+    if perc is None and decay is None:
+        decay = ncp.ones(niter)
+
     if show:
         tstart = time.time()
         print('ISTA optimization (%s thresholding)\n'
@@ -778,7 +804,21 @@ def ISTA(Op, data, niter, eps=0.1, alpha=None, eigsiter=None, eigstol=0,
         print(head1)
 
     # initialize model and cost function
-    xinv = ncp.zeros(int(Op.shape[1]), dtype=Op.dtype)
+    if x0 is None:
+        if data.ndim == 1:
+            xinv = ncp.zeros(int(Op.shape[1]), dtype=Op.dtype)
+        else:
+            xinv = ncp.zeros((int(Op.shape[1]), data.shape[1]), dtype=Op.dtype)
+    else:
+        if data.ndim != x0.ndim:
+            # error for wrong dimensions
+            raise ValueError('Number of columns of x0 and data are not the same')
+        elif x0.shape[0] != Op.shape[1]:
+            # error for wrong dimensions
+            raise ValueError('Operator and input vector have different dimensions')
+        else:
+            xinv = x0.copy()
+
     if monitorres:
         normresold = np.inf
     if returninfo:
@@ -789,7 +829,7 @@ def ISTA(Op, data, niter, eps=0.1, alpha=None, eigsiter=None, eigstol=0,
         xinvold = xinv.copy()
 
         # compute residual
-        res = data - Op.matvec(xinv)
+        res = data - Op @ xinv
         if monitorres:
             normres = np.linalg.norm(res)
             if  normres > normresold:
@@ -800,14 +840,18 @@ def ISTA(Op, data, niter, eps=0.1, alpha=None, eigsiter=None, eigstol=0,
                 normresold = normres
 
         # compute gradient
-        grad = alpha * Op.rmatvec(res)
+        grad = alpha * Op.H @ res
 
         # update inverted model
         xinv_unthesh = xinv + grad
+        if SOp is not None:
+            xinv_unthesh = SOp.H @ xinv_unthesh
         if perc is None:
-            xinv = threshf(xinv_unthesh, thresh)
+            xinv = threshf(xinv_unthesh, decay[iiter] * thresh)
         else:
             xinv = threshf(xinv_unthesh, 100 - perc)
+        if SOp is not None:
+            xinv = SOp @ xinv
 
         # model update
         xupdate = np.linalg.norm(xinv - xinvold)
@@ -851,13 +895,13 @@ def ISTA(Op, data, niter, eps=0.1, alpha=None, eigsiter=None, eigstol=0,
 
 def FISTA(Op, data, niter, eps=0.1, alpha=None, eigsiter=None, eigstol=0,
           tol=1e-10, returninfo=False, show=False, threshkind='soft',
-          perc=None, callback=None):
+          perc=None, callback=None, decay=None, SOp=None, x0=None):
     r"""Fast Iterative Shrinkage-Thresholding Algorithm (FISTA).
 
-    Solve an optimization problem with :math:`L1` regularization function given
-    the operator ``Op`` and data ``y``. The operator can be real or complex,
-    and should ideally be either square :math:`N=M` or underdetermined
-    :math:`N<M`.
+    Solve an optimization problem with :math:`L_p, \; p=0, 1/2, 1`
+    regularization, given the operator ``Op`` and data ``y``.
+    The operator can be real or complex, and should ideally be either square
+    :math:`N=M` or underdetermined :math:`N<M`.
 
     Parameters
     ----------
@@ -894,6 +938,12 @@ def FISTA(Op, data, niter, eps=0.1, alpha=None, eigsiter=None, eigstol=0,
     callback : :obj:`callable`, optional
         Function with signature (``callback(x)``) to call after each iteration
         where ``x`` is the current model vector
+    decay : :obj:`numpy.ndarray`, optional
+        Decay factor to be applied to thresholding during iterations
+    SOp : :obj:`pylops.LinearOperator`, optional
+        Regularization operator (use when solving the analysis problem)
+    x0: :obj:`numpy.ndarray`, optional
+        Initial guess
 
     Returns
     -------
@@ -922,14 +972,22 @@ def FISTA(Op, data, niter, eps=0.1, alpha=None, eigsiter=None, eigstol=0,
 
     Notes
     -----
-    Solves the following optimization problem for the operator
+    Solves the following synthesis problem for the operator
     :math:`\mathbf{Op}` and the data :math:`\mathbf{d}`:
 
     .. math::
         J = ||\mathbf{d} - \mathbf{Op} \mathbf{x}||_2^2 +
             \epsilon ||\mathbf{x}||_p
 
-    using the Fast Iterative Shrinkage-Thresholding Algorithm (FISTA) [1]_,
+    or the analysis problem:
+
+    .. math::
+        J = ||\mathbf{d} - \mathbf{Op} \mathbf{x}||_2^2 +
+            \epsilon ||\mathbf{SOp}^H\mathbf{x}||_p
+
+    if ``SOp`` is provided.
+
+    The Fast Iterative Shrinkage-Thresholding Algorithm (FISTA) [1]_ is used,
     where :math:`p=0, 1, 1/2`. This is a modified version of ISTA solver with
     improved convergence properties and limited additional computational cost.
     Similarly to the ISTA solver, the choice of the thresholding algorithm to
@@ -968,6 +1026,10 @@ def FISTA(Op, data, niter, eps=0.1, alpha=None, eigsiter=None, eigstol=0,
     # identify backend to use
     ncp = get_array_module(data)
 
+    # prepare decay (if not passed)
+    if perc is None and decay is None:
+        decay = ncp.ones(niter)
+
     if show:
         tstart = time.time()
         print('FISTA optimization (%s thresholding)\n'
@@ -1005,7 +1067,21 @@ def FISTA(Op, data, niter, eps=0.1, alpha=None, eigsiter=None, eigstol=0,
         print(head1)
 
     # initialize model and cost function
-    xinv = ncp.zeros(int(Op.shape[1]), dtype=Op.dtype)
+    if x0 is None:
+        if data.ndim == 1:
+            xinv = ncp.zeros(int(Op.shape[1]), dtype=Op.dtype)
+        else:
+            xinv = ncp.zeros((int(Op.shape[1]), data.shape[1]), dtype=Op.dtype)
+    else:
+        if data.ndim != x0.ndim:
+            # error for wrong dimensions
+            raise ValueError('Number of columns of x0 and data are not the same')
+        elif x0.shape[0] != Op.shape[1]:
+            # error for wrong dimensions
+            raise ValueError('Operator and input vector have different dimensions')
+        else:
+            xinv = x0.copy()
+
     zinv = xinv.copy()
     t = 1
     if returninfo:
@@ -1016,17 +1092,21 @@ def FISTA(Op, data, niter, eps=0.1, alpha=None, eigsiter=None, eigstol=0,
         xinvold = xinv.copy()
 
         # compute residual
-        resz = data - Op.matvec(zinv)
+        resz = data - Op @ zinv
 
         # compute gradient
-        grad = alpha * Op.rmatvec(resz)
+        grad = alpha * Op.H @ resz
 
         # update inverted model
         xinv_unthesh = zinv + grad
+        if SOp is not None:
+            xinv_unthesh = SOp.H @ xinv_unthesh
         if perc is None:
-            xinv = threshf(xinv_unthesh, thresh)
+            xinv = threshf(xinv_unthesh, decay[iiter] * thresh)
         else:
             xinv = threshf(xinv_unthesh, 100 - perc)
+        if SOp is not None:
+            xinv = SOp @ xinv
 
         # update auxiliary coefficients
         told = t
@@ -1037,7 +1117,7 @@ def FISTA(Op, data, niter, eps=0.1, alpha=None, eigsiter=None, eigstol=0,
         xupdate = np.linalg.norm(xinv - xinvold)
 
         if returninfo or show:
-            costdata = 0.5 * np.linalg.norm(data - Op.matvec(xinv)) ** 2
+            costdata = 0.5 * np.linalg.norm(data - Op @ xinv) ** 2
             costreg = eps * np.linalg.norm(xinv, ord=1)
         if returninfo:
             cost[iiter] = costdata + costreg
@@ -1280,7 +1360,7 @@ def SplitBregman(Op, RegsL1, data, niter_outer=3, niter_inner=5, RegsL2=None,
     problem:
 
     .. math::
-        J = \mu/2 ||\textbf{d} - \textbf{Op} \textbf{x} |||_2^2 +
+        J = \mu/2 ||\textbf{d} - \textbf{Op} \textbf{x}||_2^2 +
         \sum_i \epsilon_{{R}_{L2,i}}/2 ||\mathbf{d_{{R}_{L2,i}}} -
         \mathbf{R_{L2,i}} \textbf{x}||_2^2 +
         \sum_i || \textbf{y}_i ||_1 \quad s.t \quad
@@ -1291,9 +1371,9 @@ def SplitBregman(Op, RegsL1, data, niter_outer=3, niter_inner=5, RegsL2=None,
     .. math::
         (\textbf{x}^{k+1}, \textbf{y}_i^{k+1}) =
         \operatorname*{arg\,min}_{\mathbf{x}, \mathbf{y}_i}
-        ||\textbf{d} - \textbf{Op} \textbf{x} |||_2^2 +
+        ||\textbf{d} - \textbf{Op} \textbf{x}||_2^2 +
         \sum_i \epsilon_{{R}_{L2,i}}/2 ||\mathbf{d_{{R}_{L2,i}}} -
-        \mathbf{R_{L2,i}} \textbf{x}||_2^2 + \sum_i || \textbf{y}_i ||_1
+        \mathbf{R_{L2,i}} \textbf{x}||_2^2 + \sum_i || \textbf{y}_i ||_1 +
         \sum_i \epsilon_{{R}_{L1,i}}/2 ||\textbf{y}_i -
         \mathbf{R_{L1,i}} \textbf{x} - \textbf{b}_i^k||_2^2
 
